import bb.cascades 1.0
import "../tart.js" as Tart

Page {
    id: page
    objectName: "CanvasChartTest"

    property int page_ready: 0
    property int count: 0
    property bool running: false
    property variant timer: myTimer

    titleBar: TitleBar {
        title: root.activeTab.title
    }

    Container {
        topPadding: 15
        leftPadding: 35
        rightPadding: 35
        bottomPadding: 15

        background: Color.create("#ffa370")

        ScrollView {
            scrollViewProperties {
                scrollMode: ScrollMode.Horizontal
            }

            ImageView {
                id: myImageView

                onImageChanged: {
                    // print('image changed', page.count);
                    // if (page.running)
                    //     timer.start();
                }

                attachedObjects: [
                    ImageTracker {
                        id: tracker

                        onStateChanged: {
                            // print('state changed', state);
                            if (tracker.width) {
                                // print('size', tracker.width, 'x', tracker.height);
                                myImageView.image = tracker.image;
                            }
                        }

                        // onCreationCompleted: {
                        //     print('size', tracker.width, 'x', tracker.height);
                        // }
                    }
                ]
            }
        }

        WebView {
            id: webView

            visible: false
            preferredHeight: 10

            // html: "<html><head><title>Chart</title></head>" +
            //       "<body><canvas id='canvas' width='1000' height='400'></canvas></body>" +
            //       "</html>";
            url: "chart.html"

            settings {
                zoomToFitEnabled: false
                javaScriptEnabled: true
                background: Color.Transparent
            }

            onMessageReceived: {
                // print('msg', message.data.substr(0, 20));
                var msg;
                try {
                    if (message.data.substr(0, 5) == 'data:') {
                        msg = ['image', message.data];
                    }
                    else
                        msg = JSON.parse(message.data);
                } catch (error) {
                    if (message.data == 'ready') {
                        msg = ['ready'];
                    }
                    else
                        msg = ['error', error];
                }

                try {
                    switch (msg[0]) {
                        case 'ready':
                            if (++page_ready >= 2) {
                                timer.start();
                                running = true;
                                print('starting 2');
                            }
                            break;
                        case 'print':
                            console.log('CHART', msg[1]);
                            break;
                        case 'error':
                            console.log('ERROR', msg[1]);
                            break;
                        case 'image':
                            Tart.send('image', {data: msg[1]});
                            break;
                        default:
                            console.log('Error, bad msg type', msg[0]);
                            break;
                    }
                } catch (error) {
                    print('damn', error);
                }
            }
        }

        Container {
            layout: StackLayout {
                orientation: LayoutOrientation.LeftToRight
            }

            topPadding: 40

            Button {
                id: drawButton
                text: "Draw"

                onClicked: {
                    regenChart();
                }
            }

            Button {
                id: clearButton
                text: "Clear"

                onClicked: {
                    webView.evaluateJavaScript("CanvasChart.clear('canvas')");
                }
            }

            Button {
                id: grabButton
                text: "Grab"

                onClicked: {
                    webView.evaluateJavaScript("grab('canvas')");
                }
            }
        }// Container
    }// Container

    attachedObjects: [
        QTimer {
            id: myTimer
            interval: 1
            singleShot: true

            onTimeout: {
                print('timeout', ++page.count);
                regenChart();
                // grabButton.clicked();
            }
        }
    ]

    function regenChart() {
        webView.evaluateJavaScript("CanvasChart.clear('canvas')");

        // webView.postMessage(JSON.stringify(['print', [1, 'foo', 3]]));
        var dataDef = {
            title: "US Population Chart",
            xLabel: 'Year',
            yLabel: 'Population (millions)',
            labelFont: '19pt Arial',
            dataPointFont: '10pt Arial',
            renderTypes: ['lines', 'points'],
            dataPoints: [
                { x: '1790', y: Math.random() * 3 + 0 },
                { x: '1810', y: Math.random() * 10 + 5 },
                { x: '1830', y: Math.random() * 20 + 10 },
                { x: '1850', y: Math.random() * 20 + 10 },
                { x: '1870', y: Math.random() * 20 + 10 },
                { x: '1890', y: Math.random() * 40 + 20 },
                { x: '1910', y: Math.random() * 20 + 80 },
                { x: '1930', y: Math.random() * 50 + 100 },
                { x: '1950', y: Math.random() * 50 + 100 },
                { x: '1970', y: Math.random() * 100 + 150 },
                { x: '1990', y: Math.random() * 100 + 200 },
                { x: '2010', y: Math.random() * 50 + 300 }
                ]
           };

        webView.postMessage(JSON.stringify(['render', ['canvas', dataDef]]));
        // print('regen', page.count);
    }

    function onImageUpdated(data) {
        // console.log('image file updated', data.filename);
        tracker.imageSource = data.filename;
        if (page.running)
            timer.start();
        // console.log('tracker', tracker.imageSource);
    }

    function onPaneVisible(pane) {
        if (pane.objectName == page.objectName) {
            if (++page_ready >= 2) {
                timer.start();
                running = true;
                print('starting 1');
            }
        }
    }

    onCreationCompleted: {
        // myTimer.start();
        // print('creation completed', myTimer);
        Tart.Tart.onImageUpdated = onImageUpdated;

        root.paneVisible.connect(onPaneVisible);
    }
}
