'''Initialization script for BlackBerry-Tart framework.'''

import os
import sys


def _setup():
    parent = os.path.dirname(__file__)
    try:
        with open(os.path.join(parent, 'tart.cfg')) as f:
            for line in f:
                key, _, value = line.strip().partition(':')
                key = key.strip()
                value = value.strip()
                if key == 'sys.path' and value not in sys.path:
                    sys.path.append(value)
    except IOError:
        pass


def _install_slogger2():
    #~ print('installing slogger2')
    import slog2
    import ctypes

    appid = os.path.basename(os.getcwd())
    #~ print('appid', appid)

    buffers = (slog2.slog2_buffer_t * 1)()

    cfg = slog2.slog2_buffer_set_config_t()
    cfg.num_buffers = 1
    cfg.buffer_set_name = appid.encode('ascii', 'ignore')
    cfg.verbosity_level = slog2.SLOG2_INFO

    cfg.buffer_config[0].buffer_name = b"python"
    cfg.buffer_config[0].num_pages = 1

    rc = slog2.slog2_register(cfg, buffers, 0)
    #~ print('slog2_register, rc =', rc)
    sys.stdout.flush()
    # slog2.slog2c(buffers[0], 0, slog2.SLOG2_INFO, b'testing log output')

    buffer = buffers[0]
    #~ buffer = slog2.slog2_set_default_buffer(ctypes.cast(-1, slog2.slog2_buffer_t))
    #~ print('default buffer', buffer)

    class Redirector:
        def __init__(self):
            self.pending = b''
        def write(self, text):
            ascii = text.encode('ascii', 'ignore')
            while b'\n' in ascii:
                now, ascii = ascii.split(b'\n', 1)
                if self.pending:
                    now = self.pending + now
                    self.pending = b''
                slog2.slog2c(buffer, 0, slog2.SLOG2_INFO, now)
            self.pending += ascii
        def flush(self):
            pass

    sys.stderr = Redirector()
    #~ print('test stderr redirection', file=sys.stderr)
    #~ print('redirected stderr', file=sys.__stdout__)

    sys.stdout = Redirector()
    #~ sys.stdout.write('test stdout redirection\n')
    #~ print('redirected stdout', file=sys.__stdout__)

    sys.__stdout__.flush()
    sys.__stderr__.flush()


if __name__ == '__main__':
    _install_slogger2()
    _setup()

    # try:
    #     import cli
    # except ImportError:
    #     print('unable to import cli', file=sys.stderr)
    # else:
    #     import threading
    #     t = threading.Thread(target=cli.run)
    #     t.daemon = True
    #     t.start()

    try:
        import app
        a = app.App()
        a.start()

    except SystemExit:
        pass

    except:
        sys.stdout.flush()
        import traceback
        traceback.print_exc(file=sys.stderr)
        sys.stderr.flush()


# EOF
